@@ -21,6 +21,8 @@
 #include <wx/checkbox.h>
 #include <wx/choice.h>
 #include <wx/clipbrd.h>
+#include <wx/colourdata.h>
+#include <wx/colordlg.h>
 #include <wx/dataobj.h>
 #include <wx/filename.h>
 #include <wx/rawbmp.h>
@@ -33,12 +35,19 @@
 #include "NumericTextCtrl.hpp"
 #include "util.hpp"
 
-#include "../res/actual_size16.h"
-#include "../res/fit_to_screen16.h"
-#include "../res/swap_horiz16.h"
-#include "../res/swap_vert16.h"
-#include "../res/zoom_in16.h"
-#include "../res/zoom_out16.h"
+#include "../res/actual_size_dark_16.h"
+#include "../res/actual_size_light_16.h"
+#include "../res/bg16.h"
+#include "../res/fit_to_screen_dark_16.h"
+#include "../res/fit_to_screen_light_16.h"
+#include "../res/swap_horiz_dark_16.h"
+#include "../res/swap_horiz_light_16.h"
+#include "../res/swap_vert_dark_16.h"
+#include "../res/swap_vert_light_16.h"
+#include "../res/zoom_in_dark_16.h"
+#include "../res/zoom_in_light_16.h"
+#include "../res/zoom_out_dark_16.h"
+#include "../res/zoom_out_light_16.h"
 
 static REHex::ToolPanel *BitmapTool_factory(wxWindow *parent, REHex::SharedDocumentPointer &document, REHex::DocumentCtrl *document_ctrl)
 {
@@ -63,6 +72,7 @@ enum {
 	ID_ZOOM_IN,
 	ID_ZOOM_OUT,
 	ID_UPDATE_TIMER,
+	ID_BACKGROUND,
 };
 
 /* The minimum interval between updates when rendering the preview bitmap.
@@ -90,10 +100,9 @@ BEGIN_EVENT_TABLE(REHex::BitmapTool, wxPanel)
 	EVT_TOOL(ID_ACTUAL_SIZE,  REHex::BitmapTool::OnActualSize)
 	EVT_TOOL(ID_ZOOM_IN,      REHex::BitmapTool::OnZoomIn)
 	EVT_TOOL(ID_ZOOM_OUT,     REHex::BitmapTool::OnZoomOut)
+	EVT_TOOL(ID_BACKGROUND,   REHex::BitmapTool::OnBackground)
 	
 	EVT_SIZE(REHex::BitmapTool::OnSize)
-	EVT_IDLE(REHex::BitmapTool::OnIdle)
-	EVT_TIMER(ID_UPDATE_TIMER, REHex::BitmapTool::OnUpdateTimer)
 END_EVENT_TABLE()
 
 enum {
@@ -144,11 +153,7 @@ REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document,
 	image_height(256),
 	row_length(-1),
 	fit_to_screen(true),
-	actual_size(false),
-	force_bitmap_width(-1),
-	force_bitmap_height(-1),
-	bitmap_update_line(-1),
-	update_timer(this, ID_UPDATE_TIMER)
+	actual_size(false)
 {
 	wxBoxSizer *sizer = new wxBoxSizer(wxVERTICAL);
 	
@@ -222,54 +227,30 @@ REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document,
 	toolbar = new wxToolBar(this, wxID_ANY);
 	sizer->Add(toolbar, 0, wxEXPAND | wxALIGN_LEFT);
 	
-	toolbar->AddCheckTool(ID_FLIP_X, "Flip X", wxBITMAP_PNG_FROM_DATA(swap_horiz16), wxNullBitmap, "Flip X");
-	toolbar->AddCheckTool(ID_FLIP_Y, "Flip Y", wxBITMAP_PNG_FROM_DATA(swap_vert16),  wxNullBitmap, "Flip Y");
+	wxColor btn_bg_colour = wxSystemSettings::GetColour(wxSYS_COLOUR_BTNFACE);
+	bool light_buttons = Palette::is_colour_light(btn_bg_colour);
+	
+	toolbar->AddCheckTool(ID_FLIP_X, "Flip X", (light_buttons ? wxBITMAP_PNG_FROM_DATA(swap_horiz_dark_16) : wxBITMAP_PNG_FROM_DATA(swap_horiz_light_16)), wxNullBitmap, "Flip X");
+	toolbar->AddCheckTool(ID_FLIP_Y, "Flip Y", (light_buttons ? wxBITMAP_PNG_FROM_DATA(swap_vert_dark_16) : wxBITMAP_PNG_FROM_DATA(swap_vert_light_16)),  wxNullBitmap, "Flip Y");
 	
 	toolbar->AddStretchableSpace();
 	
-	toolbar->AddTool(     ID_ZOOM_IN,     "Zoom in",       wxBITMAP_PNG_FROM_DATA(zoom_in16),                     "Zoom in");
-	toolbar->AddTool(     ID_ZOOM_OUT,    "Zoom out",      wxBITMAP_PNG_FROM_DATA(zoom_out16),                    "Zoom out");
-	toolbar->AddCheckTool(ID_SCALE,       "Fit to screen", wxBITMAP_PNG_FROM_DATA(fit_to_screen16), wxNullBitmap, "Fit to screen");
-	toolbar->AddCheckTool(ID_ACTUAL_SIZE, "Actual size",   wxBITMAP_PNG_FROM_DATA(actual_size16),   wxNullBitmap, "Actual size");
+	toolbar->AddTool(     ID_BACKGROUND,  "Set background", wxBITMAP_PNG_FROM_DATA(bg16),                          "Set background");
+	toolbar->AddTool(     ID_ZOOM_IN,     "Zoom in",        (light_buttons ? wxBITMAP_PNG_FROM_DATA(zoom_in_dark_16) : wxBITMAP_PNG_FROM_DATA(zoom_in_light_16)),                           "Zoom in");
+	toolbar->AddTool(     ID_ZOOM_OUT,    "Zoom out",       (light_buttons ? wxBITMAP_PNG_FROM_DATA(zoom_out_dark_16) : wxBITMAP_PNG_FROM_DATA(zoom_out_light_16)),                         "Zoom out");
+	toolbar->AddCheckTool(ID_SCALE,       "Fit to screen",  (light_buttons ? wxBITMAP_PNG_FROM_DATA(fit_to_screen_dark_16) : wxBITMAP_PNG_FROM_DATA(fit_to_screen_light_16)), wxNullBitmap, "Fit to screen");
+	toolbar->AddCheckTool(ID_ACTUAL_SIZE, "Actual size",    (light_buttons ? wxBITMAP_PNG_FROM_DATA(actual_size_dark_16) : wxBITMAP_PNG_FROM_DATA(actual_size_light_16)),     wxNullBitmap, "Actual size");
 	
 	toolbar->ToggleTool(ID_SCALE, fit_to_screen);
 	toolbar->ToggleTool(ID_ACTUAL_SIZE, actual_size);
 	
-	bitmap_scrollwin = new wxScrolledWindow(this, wxID_ANY);
-	sizer->Add(bitmap_scrollwin, 1, wxEXPAND | wxALIGN_TOP | wxALIGN_LEFT);
+	m_preview_sizer = new wxBoxSizer(wxHORIZONTAL);
+	sizer->Add(m_preview_sizer, 1, wxEXPAND | wxALIGN_TOP | wxALIGN_LEFT);
 	
-	bitmap_scrollwin->SetScrollRate(10, 10);
+	m_preview = new Preview(this, wxSize(16, 16));
+	m_preview_sizer->Add(m_preview, 1, wxEXPAND | wxALIGN_TOP | wxALIGN_LEFT);
 	
-	bitmap = new wxBitmap(16, 16, 24);
-	
-	wxNativePixelData bmp_data(*bitmap);
-	assert(bmp_data);
-	
-	wxNativePixelData::Iterator output_ptr(bmp_data);
-	
-	for(int output_y = 0; output_y < 16; ++output_y)
-	{
-		wxNativePixelData::Iterator output_col_ptr = output_ptr;
-		
-		for(int output_x = 0; output_x < 16; ++output_x, ++output_col_ptr)
-		{
-			output_col_ptr.Red() = 0;
-			output_col_ptr.Green() = 0;
-			output_col_ptr.Blue() = 0;
-		}
-		
-		output_ptr.OffsetY(bmp_data, 1);
-	}
-	
-	wxSizer *scrollwin_sizer = new wxBoxSizer(wxVERTICAL);
-	
-	s_bitmap = new wxGenericStaticBitmap(bitmap_scrollwin, wxID_ANY, *bitmap);
-	scrollwin_sizer->Add(s_bitmap);
-	
-	bitmap_scrollwin->SetSizer(scrollwin_sizer);
-	bitmap_scrollwin->FitInside();
-	
-	s_bitmap->Bind(wxEVT_RIGHT_DOWN, &REHex::BitmapTool::OnBitmapRightDown, this);
+	m_preview->Bind(wxEVT_RIGHT_DOWN, &REHex::BitmapTool::OnBitmapRightDown, this);
 	
 	SetSizerAndFit(sizer);
 	toolbar->Realize();
@@ -279,10 +260,6 @@ REHex::BitmapTool::BitmapTool(wxWindow *parent, SharedDocumentPointer &document,
 	update();
 }
 
-REHex::BitmapTool::~BitmapTool() {
-	delete bitmap;
-}
-
 std::string REHex::BitmapTool::name() const
 {
 	return "BitmapTool";
@@ -380,8 +357,6 @@ void REHex::BitmapTool::reset_row_length_spinner()
 
 void REHex::BitmapTool::update()
 {
-	bitmap_update_line = -1;
-	
 	try {
 		image_offset = offset_textctrl->GetValue<BitOffset>(BitOffset::ZERO);
 	}
@@ -405,12 +380,7 @@ void REHex::BitmapTool::update()
 	fit_to_screen = toolbar->GetToolState(ID_SCALE);
 	actual_size = toolbar->GetToolState(ID_ACTUAL_SIZE);
 	
-	if(force_bitmap_width >= 0 && force_bitmap_height >= 0)
-	{
-		bitmap_width = force_bitmap_width;
-		bitmap_height = force_bitmap_height;
-	}
-	else if(actual_size)
+	if(actual_size)
 	{
 		bitmap_width = image_width;
 		bitmap_height = image_height;
@@ -422,17 +392,16 @@ void REHex::BitmapTool::update()
 		/* Figure out how much space is available for the wxStaticBitmap preview.
 		 *
 		 * At the point we are called, our sizer hasn't been resized or updated the layout
-		 * of our child windows. So we need to get the current size of the wxScrolledWindow
-		 * that contains the wxStaticBitmap, and then adjust it by the difference between
-		 * our size and that of the sizer to find what its size will be after resizing
-		 * completes.
+		 * of our child windows. So we need to get the current size of the sizer containing
+		 * the preview, and then adjust it by the difference between our size and that of
+		 * the sizer to find what its size will be after resizing completes.
 		*/
 		
 		wxSize this_size  = GetSize();
 		wxSize sizer_size = GetSizer()->GetSize();
 		
-		int max_w = bitmap_scrollwin->GetSize().GetWidth()  - (sizer_size.GetWidth()  - this_size.GetWidth());
-		int max_h = bitmap_scrollwin->GetSize().GetHeight() - (sizer_size.GetHeight() - this_size.GetHeight());
+		int max_w = m_preview_sizer->GetSize().GetWidth()  - (sizer_size.GetWidth()  - this_size.GetWidth());
+		int max_h = m_preview_sizer->GetSize().GetHeight() - (sizer_size.GetHeight() - this_size.GetHeight());
 		
 		/* Scale the image to fit the available width/height, while preserving aspect ratio. */
 		
@@ -470,26 +439,8 @@ void REHex::BitmapTool::update()
 	toolbar->EnableTool(ID_ZOOM_IN,  ZOOM_LEVELS[LAST_ZOOM_LEVEL_IDX] > zoom);
 	toolbar->EnableTool(ID_ZOOM_OUT, ZOOM_LEVELS[0] < zoom);
 	
-	bitmap_lines_per_idle = (bitmap_width > 1024) ? 20 : 200;
-	
-	wxBitmap *new_bitmap = new wxBitmap(bitmap_width, bitmap_height, 24);
-	s_bitmap->SetBitmap(*new_bitmap);
-	
-	delete bitmap;
-	bitmap = new_bitmap;
-	
-	render_region(0, bitmap_lines_per_idle, image_offset, image_width, image_height);
-	
-	if(bitmap_lines_per_idle < bitmap_height)
-	{
-		bitmap_update_line = bitmap_lines_per_idle;
-		update_timer.Start(UPDATE_TIMER_MS, wxTIMER_ONE_SHOT);
-	}
-	else{
-		update_timer.Stop();
-	}
-	
-	bitmap_scrollwin->SetVirtualSize(s_bitmap->GetSize());
+	m_preview->set_bitmap_size(wxSize(bitmap_width, bitmap_height));
+	GetSizer()->Layout();
 }
 
 /* The following functions take a uint32_t value and extract an X bit wide value whose least
@@ -780,36 +731,40 @@ void REHex::BitmapTool::update_pixel_fmt()
 	}
 }
 
-void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offset, int width, int height)
+template<typename PDT> void REHex::BitmapTool::render_rect(wxBitmap *bitmap, const wxRect &image_rect, bool blend_bg)
 {
 	int output_width = bitmap->GetWidth();
 	int output_height = bitmap->GetHeight();
+
+	int width = image_width;
+	int height = image_height;
 	
 	bool flip_x = toolbar->GetToolState(ID_FLIP_X);
 	bool flip_y = toolbar->GetToolState(ID_FLIP_Y);
 	
 	/* Read in the image data, convert the source pixel format and write it to the wxBitmap. */
 	
-	wxNativePixelData bmp_data(*bitmap);
+	PDT bmp_data(*bitmap);
 	assert(bmp_data);
 	
 	std::vector<unsigned char> data;
 	BitOffset data_begin = BitOffset::ZERO, data_end = BitOffset::ZERO;
 	
-	wxNativePixelData::Iterator output_ptr(bmp_data);
-	output_ptr.OffsetY(bmp_data, region_y);
+	typename PDT::Iterator output_ptr(bmp_data);
 	
-	for(int output_y = region_y; output_y < output_height && output_y < region_y + region_h; ++output_y)
+	for(int output_y = 0; output_y < output_height; ++output_y)
 	{
 		int input_y = output_y;
 		
-		if(height != output_height)
+		if(image_rect.GetHeight() != bitmap->GetHeight())
 		{
-			input_y = (double)(input_y) * ((double)(height) / (double)(output_height));
+			input_y = (double)(input_y) * ((double)(image_rect.GetHeight()) / (double)(bitmap->GetHeight()));
 			
 			input_y = std::max(input_y, 0);
 			input_y = std::min(input_y, height - 1);
 		}
+
+		input_y += image_rect.y;
 		
 		if(flip_y)
 		{
@@ -821,8 +776,8 @@ void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offs
 			: (width * pixel_fmt_multi) / pixel_fmt_div;
 		
 		BitOffset line_off = row_length > 0
-			? offset + BitOffset((input_y * line_len), 0)
-			: offset + BitOffset(((width * input_y * pixel_fmt_multi) / pixel_fmt_div), 0);
+			? image_offset + BitOffset((input_y * line_len), 0)
+			: image_offset + BitOffset(((width * input_y * pixel_fmt_multi) / pixel_fmt_div), 0);
 		
 		if(line_off < data_begin || (line_off + BitOffset(line_len, 0)) > data_end)
 		{
@@ -848,36 +803,27 @@ void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offs
 		assert((line_off - data_begin).byte_aligned());
 		off_t data_line_offset = (line_off - data_begin).byte();
 		
-		wxNativePixelData::Iterator output_col_ptr = output_ptr;
+		typename PDT::Iterator output_col_ptr = output_ptr;
 		
 		for(int output_x = 0; output_x < output_width; ++output_x, ++output_col_ptr)
 		{
 			int input_x = output_x;
 			
-			if(width != output_width)
+			if(image_rect.GetWidth() != bitmap->GetWidth())
 			{
-				input_x = (double)(input_x) * ((double)(width) / (double)(output_width));
+				input_x = (double)(input_x) * ((double)(image_rect.GetWidth()) / (double)(bitmap->GetWidth()));
 				
 				input_x = std::max(input_x, 0);
 				input_x = std::min(input_x, width - 1);
 			}
 			
+			input_x += image_rect.x;
+			
 			if(flip_x)
 			{
 				input_x = ((width - 1) - input_x);
 			}
 			
-			/* Initialise output to chequerboard pattern. */
-			
-			bool x_even = (output_x % 20) >= 10;
-			bool y_even = (output_y % 20) >= 10;
-			
-			int chequerboard_colour = (x_even ^ y_even) ? 0x66 : 0x99;
-			
-			output_col_ptr.Red()   = chequerboard_colour;
-			output_col_ptr.Green() = chequerboard_colour;
-			output_col_ptr.Blue()  = chequerboard_colour;
-			
 			if((off_t)(data.size()) <= data_line_offset)
 			{
 				continue;
@@ -955,23 +901,36 @@ void REHex::BitmapTool::render_region(int region_y, int region_h, BitOffset offs
 			
 			wxColour colour = colour_fmt_conv(rgb);
 			
-			if(colour.Alpha() != wxALPHA_OPAQUE)
+			if(blend_bg)
 			{
-				/* Blend colours with an alpha channel into the chequerboard. */
-				
-				double alpha = (double)(colour.Alpha()) / 255.0;
-				
-				int red   = ((double)(colour.Red())   * alpha) + ((double)(output_col_ptr.Red())   * (1.0 - alpha));
-				int green = ((double)(colour.Green()) * alpha) + ((double)(output_col_ptr.Green()) * (1.0 - alpha));
-				int blue  = ((double)(colour.Blue())  * alpha) + ((double)(output_col_ptr.Blue())  * (1.0 - alpha));
-				
-				colour.Set(red, green, blue);
+				if(colour.Alpha() != wxALPHA_OPAQUE)
+				{
+					/* Blend colours with an alpha channel into the chequerboard. */
+					
+					double alpha = (double)(colour.Alpha()) / 255.0;
+					
+					int red   = ((double)(colour.Red())   * alpha) + ((double)(output_col_ptr.Red())   * (1.0 - alpha));
+					int green = ((double)(colour.Green()) * alpha) + ((double)(output_col_ptr.Green()) * (1.0 - alpha));
+					int blue  = ((double)(colour.Blue())  * alpha) + ((double)(output_col_ptr.Blue())  * (1.0 - alpha));
+					
+					colour.Set(red, green, blue);
+					
+				}
 				
+				output_col_ptr.Red()   = colour.Red();
+				output_col_ptr.Green() = colour.Green();
+				output_col_ptr.Blue()  = colour.Blue();
+			}
+			else{
+				output_col_ptr.Red()   = colour.Red();
+				output_col_ptr.Green() = colour.Green();
+				output_col_ptr.Blue()  = colour.Blue();
+				
+				if(bitmap->HasAlpha())
+				{
+					output_col_ptr.Alpha() = colour.Alpha();
+				}
 			}
-			
-			output_col_ptr.Red()   = colour.Red();
-			output_col_ptr.Green() = colour.Green();
-			output_col_ptr.Blue()  = colour.Blue();
 		}
 		
 		output_ptr.OffsetY(bmp_data, 1);
@@ -1105,6 +1064,36 @@ void REHex::BitmapTool::OnZoomOut(wxCommandEvent &event)
 	}
 }
 
+void REHex::BitmapTool::OnBackground(wxCommandEvent &event)
+{
+	wxMenu menu;
+	
+	wxMenuItem *none = menu.Append(wxID_ANY, "None");
+	menu.Bind(wxEVT_MENU, [&](wxCommandEvent &event)
+	{
+		m_preview->set_bg(wxNullColour);
+	}, none->GetId(), none->GetId());
+	
+	wxMenuItem *colour = menu.Append(wxID_ANY, "Choose colour...");
+	menu.Bind(wxEVT_MENU, [&](wxCommandEvent &event)
+	{
+		wxColourData cd;
+		
+		#if wxCHECK_VERSION(3,1,0)
+		cd.SetChooseAlpha(false);
+		#endif
+
+		wxColourDialog dlg(this, &cd);
+
+		if(dlg.ShowModal() == wxID_OK)
+		{
+			m_preview->set_bg(dlg.GetColourData().GetColour());
+		}
+	}, colour->GetId(), colour->GetId());
+	
+	PopupMenu(&menu);
+}
+
 void REHex::BitmapTool::OnXXX(wxCommandEvent &event)
 {
 	if(event.GetEventObject() == offset_textctrl)
@@ -1127,62 +1116,24 @@ void REHex::BitmapTool::OnSize(wxSizeEvent &event)
 	event.Skip();
 }
 
-void REHex::BitmapTool::OnIdle(wxIdleEvent &event)
-{
-	if(bitmap_update_line >= 0)
-	{
-		render_region(bitmap_update_line, bitmap_lines_per_idle, image_offset, image_width, image_height);
-		bitmap_update_line += bitmap_lines_per_idle;
-		
-		s_bitmap->Refresh();
-		
-		if(bitmap_update_line >= bitmap_height)
-		{
-			update_timer.Stop();
-			bitmap_update_line = -1;
-		}
-		else{
-			update_timer.Start(UPDATE_TIMER_MS, wxTIMER_ONE_SHOT);
-			event.RequestMore();
-		}
-	}
-}
-
-void REHex::BitmapTool::OnUpdateTimer(wxTimerEvent &event)
-{
-	if(bitmap_update_line >= 0)
-	{
-		render_region(bitmap_update_line, bitmap_lines_per_idle, image_offset, image_width, image_height);
-		bitmap_update_line += bitmap_lines_per_idle;
-		
-		s_bitmap->Refresh();
-		
-		if(bitmap_update_line >= bitmap_height)
-		{
-			update_timer.Stop();
-			bitmap_update_line = -1;
-		}
-		else{
-			update_timer.Start(UPDATE_TIMER_MS, wxTIMER_ONE_SHOT);
-		}
-	}
-}
-
 void REHex::BitmapTool::OnBitmapRightDown(wxMouseEvent &event)
 {
 	wxMenu menu;
 	
-	wxMenuItem *copy = menu.Append(wxID_ANY, "&Copy preview image");
+	wxMenuItem *copy = menu.Append(wxID_ANY, "&Copy image");
 	menu.Bind(wxEVT_MENU, [&](wxCommandEvent &event)
 	{
 		ClipboardGuard cg;
 		if(cg)
 		{
-			wxTheClipboard->SetData(new wxBitmapDataObject(*bitmap));
+			wxBitmap fullsize_bitmap(image_width, image_height, 32);
+			render_rect<wxAlphaPixelData>(&fullsize_bitmap, wxRect(0, 0, image_width, image_height), false);
+			
+			wxTheClipboard->SetData(new wxBitmapDataObject(fullsize_bitmap));
 		}
 	}, copy->GetId(), copy->GetId());
 	
-	wxMenuItem *save = menu.Append(wxID_ANY, "&Save preview image");
+	wxMenuItem *save = menu.Append(wxID_ANY, "&Save image");
 	menu.Bind(wxEVT_MENU, [&](wxCommandEvent &event)
 	{
 		CallAfter([&]()
@@ -1227,7 +1178,10 @@ void REHex::BitmapTool::OnBitmapRightDown(wxMouseEvent &event)
 				abort();
 			}
 			
-			bool save_ok = bitmap->SaveFile(save_fn.GetFullPath(), type);
+			wxBitmap fullsize_bitmap(image_width, image_height, 32);
+			render_rect<wxAlphaPixelData>(&fullsize_bitmap, wxRect(0, 0, image_width, image_height), false);
+			
+			bool save_ok = fullsize_bitmap.SaveFile(save_fn.GetFullPath(), type);
 			if(!save_ok)
 			{
 				wxMessageBox("Unable to save image", "Error", (wxOK | wxICON_ERROR | wxCENTRE), this);
@@ -1235,12 +1189,6 @@ void REHex::BitmapTool::OnBitmapRightDown(wxMouseEvent &event)
 		});
 	}, save->GetId(), save->GetId());
 	
-	if(bitmap_update_line >= 0)
-	{
-		copy->Enable(false);
-		save->Enable(false);
-	}
-	
 	PopupMenu(&menu);
 }
 
@@ -1256,6 +1204,11 @@ void REHex::BitmapTool::set_image_size(int width, int height)
 	height_textctrl->SetValue(height);
 }
 
+wxSize REHex::BitmapTool::get_image_size() const
+{
+	return wxSize(image_width, image_height);
+}
+
 void REHex::BitmapTool::set_pixel_format(PixelFormat format)
 {
 	switch(format)
@@ -1309,24 +1262,6 @@ void REHex::BitmapTool::set_pixel_format(PixelFormat format)
 	update();
 }
 
-void REHex::BitmapTool::force_bitmap_size(int width, int height)
-{
-	force_bitmap_width = width;
-	force_bitmap_height = height;
-	
-	update();
-}
-
-bool REHex::BitmapTool::is_processing()
-{
-	return bitmap_update_line >= 0;
-}
-
-wxBitmap REHex::BitmapTool::get_bitmap()
-{
-	return *bitmap;
-}
-
 void REHex::BitmapTool::set_flip_x(bool flip_x)
 {
 	toolbar->ToggleTool(ID_FLIP_X, flip_x);
@@ -1341,4 +1276,87 @@ void REHex::BitmapTool::set_row_length(int row_length)
 {
 	row_packed_cb->SetValue(false);
 	row_length_spinner->SetValue(row_length);
+	this->row_length = row_length;
+}
+
+REHex::BitmapTool::Preview::Preview(BitmapTool *parent, const wxSize &size):
+	ProceduralBitmap(parent, wxID_ANY, size),
+	m_parent(parent),
+	m_bg(wxNullColour) {}
+
+void REHex::BitmapTool::Preview::set_bg(const wxColour &bg)
+{
+	m_bg = bg;
+	Refresh();
+}
+
+wxBitmap REHex::BitmapTool::Preview::render_rect(const wxRect &rect)
+{
+	wxSize image_size = m_parent->get_image_size();
+	wxSize bitmap_size = get_bitmap_size();
+	
+	double ws = (double)(image_size.GetWidth()) / (double)(bitmap_size.GetWidth());
+	double hs = (double)(image_size.GetHeight()) / (double)(bitmap_size.GetHeight());
+	
+	int ix = (double)(rect.x) * ws;
+	ix = std::max(0, ix);
+	ix = std::min(ix, (image_size.GetWidth() - 1));
+	
+	int iw = (double)(rect.width) * ws;
+	iw = std::max(1, iw);
+	iw = std::min(iw, (image_size.GetWidth() - ix));
+	
+	int iy = (double)(rect.y) * hs;
+	iy = std::max(0, iy);
+	iy = std::min(iy, (image_size.GetHeight() - 1));
+	
+	int ih = (double)(rect.height) * hs;
+	ih = std::max(1, ih);
+	ih = std::min(ih, (image_size.GetHeight() - iy));
+	
+	wxRect image_rect(ix, iy, iw, ih);
+	
+	wxBitmap bitmap(rect.GetSize(), wxBITMAP_SCREEN_DEPTH);
+	
+	fill_bg(&bitmap, rect.x, rect.y);
+	m_parent->render_rect<wxNativePixelData>(&bitmap, image_rect, true);
+	
+	return bitmap;
+}
+
+void REHex::BitmapTool::Preview::fill_bg(wxBitmap *bitmap, int base_x, int base_y)
+{
+	wxNativePixelData bmp_data(*bitmap);
+	assert(bmp_data);
+	
+	wxNativePixelData::Iterator output_ptr(bmp_data);
+	
+	for(int y = 0; y < bitmap->GetHeight(); ++y)
+	{
+		wxNativePixelData::Iterator output_col_ptr = output_ptr;
+		
+		for(int x = 0; x < bitmap->GetWidth(); ++x, ++output_col_ptr)
+		{
+			if(m_bg.IsOk())
+			{
+				output_col_ptr.Red() = m_bg.Red();
+				output_col_ptr.Green() = m_bg.Green();
+				output_col_ptr.Blue() = m_bg.Blue();
+			}
+			else{
+				/* Initialise output to chequerboard pattern. */
+				
+				bool x_even = ((base_x + x) % 20) >= 10;
+				bool y_even = ((base_y + y) % 20) >= 10;
+				
+				int chequerboard_colour = (x_even ^ y_even) ? 0x66 : 0x99;
+				
+				output_col_ptr.Red()   = chequerboard_colour;
+				output_col_ptr.Green() = chequerboard_colour;
+				output_col_ptr.Blue()  = chequerboard_colour;
+			}
+		}
+
+		output_ptr.OffsetY(bmp_data, 1);
+	}
 }
