@@ -1,5 +1,5 @@
 /* Reverse Engineer's Hex Editor
- * Copyright (C) 2019-2024 Daniel Collins <solemnwarning@solemnwarning.net>
+ * Copyright (C) 2019-2025 Daniel Collins <solemnwarning@solemnwarning.net>
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License version 2 as published by
@@ -23,6 +23,7 @@
 #include "../src/document.hpp"
 #include "../src/search.hpp"
 #include "../src/SharedDocumentPointer.hpp"
+#include "testutil.hpp"
 
 /* This MUST come after the wxWidgets headers have been included, else we pull in windows.h BEFORE the wxWidgets
  * headers when building on Windows and this causes unicode-flavoured pointer conversion errors.
@@ -41,10 +42,10 @@ static int64_t htobe64s(int64_t h) { return htobe64(h); }
 
 TEST(SearchValue, SearchForU8)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("100", REHex::Search::Value::FMT_I8);
 	
 	{
@@ -70,10 +71,10 @@ TEST(SearchValue, SearchForU8)
 
 TEST(SearchValue, SearchForS8)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-100", REHex::Search::Value::FMT_I8);
 	
 	{
@@ -99,10 +100,10 @@ TEST(SearchValue, SearchForS8)
 
 TEST(SearchValue, SearchForU16LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("50000", REHex::Search::Value::FMT_I16 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -140,10 +141,10 @@ TEST(SearchValue, SearchForU16LE)
 
 TEST(SearchValue, SearchForS16LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-2000", REHex::Search::Value::FMT_I16 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -181,10 +182,10 @@ TEST(SearchValue, SearchForS16LE)
 
 TEST(SearchValue, SearchForU16BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("50000", REHex::Search::Value::FMT_I16 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -232,10 +233,10 @@ TEST(SearchValue, SearchForU16BE)
 
 TEST(SearchValue, SearchForS16BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-2000", REHex::Search::Value::FMT_I16 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -283,10 +284,10 @@ TEST(SearchValue, SearchForS16BE)
 
 TEST(SearchValue, SearchForU16EE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1234", REHex::Search::Value::FMT_I16 | REHex::Search::Value::FMT_LE | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -302,10 +303,10 @@ TEST(SearchValue, SearchForU16EE)
 
 TEST(SearchValue, SearchForU32LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("4000000000", REHex::Search::Value::FMT_I32 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -348,10 +349,10 @@ TEST(SearchValue, SearchForU32LE)
 
 TEST(SearchValue, SearchForS32LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-1000000000", REHex::Search::Value::FMT_I32 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -394,10 +395,10 @@ TEST(SearchValue, SearchForS32LE)
 
 TEST(SearchValue, SearchForU32BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("4000000000", REHex::Search::Value::FMT_I32 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -445,10 +446,10 @@ TEST(SearchValue, SearchForU32BE)
 
 TEST(SearchValue, SearchForS32BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-1000000000", REHex::Search::Value::FMT_I32 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -496,10 +497,10 @@ TEST(SearchValue, SearchForS32BE)
 
 TEST(SearchValue, SearchForU32EE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1234", REHex::Search::Value::FMT_I32 | REHex::Search::Value::FMT_LE | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -515,10 +516,10 @@ TEST(SearchValue, SearchForU32EE)
 
 TEST(SearchValue, SearchForU64LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("8000000000", REHex::Search::Value::FMT_I64 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -566,10 +567,10 @@ TEST(SearchValue, SearchForU64LE)
 
 TEST(SearchValue, SearchForS64LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-8000000000", REHex::Search::Value::FMT_I64 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -617,10 +618,10 @@ TEST(SearchValue, SearchForS64LE)
 
 TEST(SearchValue, SearchForU64BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("8000000000", REHex::Search::Value::FMT_I64 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -668,10 +669,10 @@ TEST(SearchValue, SearchForU64BE)
 
 TEST(SearchValue, SearchForS64BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("-8000000000", REHex::Search::Value::FMT_I64 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -719,10 +720,10 @@ TEST(SearchValue, SearchForS64BE)
 
 TEST(SearchValue, SearchForU64EE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1234", REHex::Search::Value::FMT_I64 | REHex::Search::Value::FMT_LE | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -738,10 +739,10 @@ TEST(SearchValue, SearchForU64EE)
 
 TEST(SearchValue, SearchFor1664EE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("123",
 		REHex::Search::Value::FMT_I16 | REHex::Search::Value::FMT_I64 |
 		REHex::Search::Value::FMT_LE | REHex::Search::Value::FMT_BE);
@@ -779,10 +780,10 @@ TEST(SearchValue, SearchFor1664EE)
 
 TEST(SearchValue, SearchForF32LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1.5", REHex::Search::Value::FMT_F32 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -835,10 +836,10 @@ TEST(SearchValue, SearchForF32LE)
 
 TEST(SearchValue, SearchForF32Infinity)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("inf", REHex::Search::Value::FMT_F32 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -871,10 +872,10 @@ TEST(SearchValue, SearchForF32Infinity)
 
 TEST(SearchValue, SearchForF32NaN)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("NaN", REHex::Search::Value::FMT_F32 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -900,10 +901,10 @@ TEST(SearchValue, SearchForF32NaN)
 
 TEST(SearchValue, SearchForF32BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1.5", REHex::Search::Value::FMT_F32 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -934,10 +935,10 @@ TEST(SearchValue, SearchForF32BE)
 
 TEST(SearchValue, SearchForF32EE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1.5", REHex::Search::Value::FMT_F32 | REHex::Search::Value::FMT_LE | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -978,10 +979,10 @@ TEST(SearchValue, SearchForF32EE)
 
 TEST(SearchValue, SearchForF64LE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1.5", REHex::Search::Value::FMT_F64 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -1034,10 +1035,10 @@ TEST(SearchValue, SearchForF64LE)
 
 TEST(SearchValue, SearchForF64Infinity)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("inf", REHex::Search::Value::FMT_F64 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -1070,10 +1071,10 @@ TEST(SearchValue, SearchForF64Infinity)
 
 TEST(SearchValue, SearchForF64NaN)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("NaN", REHex::Search::Value::FMT_F64 | REHex::Search::Value::FMT_LE);
 	
 	{
@@ -1099,10 +1100,10 @@ TEST(SearchValue, SearchForF64NaN)
 
 TEST(SearchValue, SearchForF64BE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1.5", REHex::Search::Value::FMT_F64 | REHex::Search::Value::FMT_BE);
 	
 	{
@@ -1133,10 +1134,10 @@ TEST(SearchValue, SearchForF64BE)
 
 TEST(SearchValue, SearchForF64EE)
 {
-	wxFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
+	AutoFrame frame(NULL, wxID_ANY, wxT("Unit tests"));
 	REHex::SharedDocumentPointer doc(REHex::SharedDocumentPointer::make());
 	
-	REHex::Search::Value s(&frame, doc);
+	REHex::Search::Value s(frame, doc);
 	s.configure("1.5", REHex::Search::Value::FMT_F64 | REHex::Search::Value::FMT_LE | REHex::Search::Value::FMT_BE);
 	
 	{
