@@ -71,12 +71,12 @@ bool wxLuaDataObjectSimple::GetDataHere(void* buf) const
         int nOldTop = m_wxlState.lua_GetTop();
         m_wxlState.wxluaT_PushUserDataType(this, wxluatype_wxLuaDataObjectSimple, true);
 
-        if (m_wxlState.LuaPCall(0, 2) == 0)
+        if (m_wxlState.LuaPCall(1, 2) == 0)
         {
             result = m_wxlState.GetBooleanType(-2);
 
-            const void *lua_buf = m_wxlState.lua_ToString(-1);
-            size_t len = (size_t)m_wxlState.lua_StrLen(-1);
+            size_t len;
+            const void *lua_buf = (const void *)wxlua_getstringtypelen(m_wxlState.GetLuaState(), -1, &len);
 
             memcpy(buf, lua_buf, len);
         }
